@@ -1,6 +1,6 @@
 package IO::String;
 
-# Copyright 1998-2003 Gisle Aas.
+# Copyright 1998-2004 Gisle Aas.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
@@ -8,7 +8,7 @@ package IO::String;
 require 5.005_03;
 use strict;
 use vars qw($VERSION $DEBUG $IO_CONSTANTS);
-$VERSION = "1.04";  # $Date: 2004/01/08 12:41:23 $
+$VERSION = "1.06";  # $Date: 2004/11/05 15:05:27 $
 
 use Symbol ();
 
@@ -18,7 +18,7 @@ sub new
     my $self = bless Symbol::gensym(), ref($class) || $class;
     tie *$self, $self;
     $self->open(@_);
-    $self;
+    return $self;
 }
 
 sub open
@@ -30,13 +30,14 @@ sub open
 	my $bufref = ref($_[0]) ? $_[0] : \$_[0];
 	$$bufref = "" unless defined $$bufref;
 	*$self->{buf} = $bufref;
-    } else {
+    }
+    else {
 	my $buf = "";
 	*$self->{buf} = \$buf;
     }
     *$self->{pos} = 0;
     *$self->{lno} = 0;
-    $self;
+    return $self;
 }
 
 sub pad
@@ -45,7 +46,7 @@ sub pad
     my $old = *$self->{pad};
     *$self->{pad} = substr($_[0], 0, 1) if @_;
     return "\0" unless defined($old) && length($old);
-    $old;
+    return $old;
 }
 
 sub dump
@@ -54,6 +55,7 @@ sub dump
     my $self = shift;
     print Data::Dumper->Dump([$self], ['*self']);
     print Data::Dumper->Dump([*$self{HASH}], ['$self{HASH}']);
+    return;
 }
 
 sub TIEHANDLE
@@ -63,7 +65,7 @@ sub TIEHANDLE
     my $class = shift;
     my $self = bless Symbol::gensym(), $class;
     $self->open(@_);
-    $self;
+    return $self;
 }
 
 sub DESTROY
@@ -84,13 +86,21 @@ sub close
 	untie *$self;
 	undef *$self;
     }
-    $self;
+    return 1;
 }
 
 sub opened
 {
     my $self = shift;
-    defined *$self->{buf};
+    return defined *$self->{buf};
+}
+
+sub binmode
+{
+    my $self = shift;
+    return 1 unless @_;
+    # XXX don't know much about layers yet :-(
+    return 0;
 }
 
 sub getc
@@ -104,13 +114,14 @@ sub getc
 sub ungetc
 {
     my $self = shift;
-    $self->setpos($self->getpos() - 1)
+    $self->setpos($self->getpos() - 1);
+    return 1;
 }
 
 sub eof
 {
     my $self = shift;
-    length(${*$self->{buf}}) <= *$self->{pos};
+    return length(${*$self->{buf}}) <= *$self->{pos};
 }
 
 sub print
@@ -119,16 +130,20 @@ sub print
     if (defined $\) {
 	if (defined $,) {
 	    $self->write(join($,, @_).$\);
-	} else {
+	}
+	else {
 	    $self->write(join("",@_).$\);
 	}
-    } else {
+    }
+    else {
 	if (defined $,) {
 	    $self->write(join($,, @_));
-	} else {
+	}
+	else {
 	    $self->write(join("",@_));
 	}
     }
+    return 1;
 }
 *printflush = \*print;
 
@@ -138,6 +153,7 @@ sub printf
     print "PRINTF(@_)\n" if $DEBUG;
     my $fmt = shift;
     $self->write(sprintf($fmt, @_));
+    return 1;
 }
 
 
@@ -150,7 +166,8 @@ sub _init_seek_constants
 	$SEEK_SET = &IO::Handle::SEEK_SET;
 	$SEEK_CUR = &IO::Handle::SEEK_CUR;
 	$SEEK_END = &IO::Handle::SEEK_END;
-    } else {
+    }
+    else {
 	$SEEK_SET = 0;
 	$SEEK_CUR = 1;
 	$SEEK_END = 2;
@@ -161,7 +178,7 @@ sub _init_seek_constants
 sub seek
 {
     my($self,$off,$whence) = @_;
-    my $buf = *$self->{buf} || return;
+    my $buf = *$self->{buf} || return 0;
     my $len = length($$buf);
     my $pos = *$self->{pos};
 
@@ -170,12 +187,13 @@ sub seek
     if    ($whence == $SEEK_SET) { $pos = $off }
     elsif ($whence == $SEEK_CUR) { $pos += $off }
     elsif ($whence == $SEEK_END) { $pos = $len + $off }
-    else { die "Bad whence ($whence)" }
+    else                         { die "Bad whence ($whence)" }
     print "SEEK(POS=$pos,OFF=$off,LEN=$len)\n" if $DEBUG;
 
     $pos = 0 if $pos < 0;
     $self->truncate($pos) if $pos > $len;  # extend file
     *$self->{pos} = $pos;
+    return 1;
 }
 
 sub pos
@@ -189,7 +207,7 @@ sub pos
 	$pos = $len if $pos > $len;
 	*$self->{pos} = $pos;
     }
-    $old;
+    return $old;
 }
 
 sub getpos { shift->pos; }
@@ -221,10 +239,15 @@ sub getline
 	while (defined($c = $self->getc)) {
 	    if ($c eq "\n") {
 		$eol++;
-	    } elsif ($eol > 1) {
+		next if $eol > 2;
+	    }
+	    elsif ($eol > 1) {
 		$self->ungetc($c);
 		last;
 	    }
+	    else {
+		$eol = 0;
+	    }
 	    $para .= $c;
 	}
 	return $para;   # XXX wantarray
@@ -263,7 +286,7 @@ sub input_line_number
     my $self = shift;
     my $old = *$self->{lno};
     *$self->{lno} = shift if @_;
-    $old;
+    return $old;
 }
 
 sub truncate
@@ -274,10 +297,11 @@ sub truncate
     if (length($$buf) >= $len) {
 	substr($$buf, $len) = '';
 	*$self->{pos} = $len if $len < *$self->{pos};
-    } else {
+    }
+    else {
 	$$buf .= ($self->pad x ($len - length($$buf)));
     }
-    $self;
+    return 1;
 }
 
 sub read
@@ -292,7 +316,8 @@ sub read
     $len = $rem if $len > $rem;
     if (@_ > 2) { # read offset
 	substr($_[0],$_[2]) = substr($$buf, $pos, $len);
-    } else {
+    }
+    else {
 	$_[0] = substr($$buf, $pos, $len);
     }
     *$self->{pos} += $len;
@@ -324,7 +349,7 @@ sub write
     }
     substr($$buf, $pos, $len) = substr($_[0], $off, $len);
     *$self->{pos} += $len;
-    $len;
+    return $len;
 }
 
 *sysread = \&read;
@@ -361,7 +386,7 @@ sub blocking {
     my $self = shift;
     my $old = *$self->{blocking} || 0;
     *$self->{blocking} = shift if @_;
-    $old;
+    return $old;
 }
 
 my $notmuch = sub { return };
@@ -388,14 +413,13 @@ my $notmuch = sub { return };
 *TELL   = \&getpos;
 *EOF    = \&eof;
 *CLOSE  = \&close;
-
-*BINMODE = $notmuch;
+*BINMODE = \&binmode;
 
 
 sub string_ref
 {
     my $self = shift;
-    *$self->{buf};
+    return *$self->{buf};
 }
 *sref = \&string_ref;
 
